# imports
import main as ws
import plotly.graph_objects as go

# title page
title_page = {
    'img_url': '',
    'title': 'Title of Title Page',
    'summary': {'Summary item 1': 'item text 1', 'Summary item 2': 'item text 2'},
    'footer': ['- use custom title image via the img_url parameter', '- footer2']
}


# simple plotly fig
def simple_fig():
    x = list(range(1, 11))
    y1 = [2 * i for i in x]
    y2 = [3 * i for i in x]
    trace1 = go.Scatter(x=x, y=y1, mode='markers+lines', name='Line 1')
    trace2 = go.Scatter(x=x, y=y2, mode='markers+lines', name='Line 2')
    fig = go.Figure()
    fig.add_trace(trace1)
    fig.add_trace(trace2)
    fig.update_layout(title='Simple Plotly Line Figure with Two Lines')
    return fig


# content pages
content = {
    'Topcat A': {
        'Subcat X': [
            {
                'title': 'Page Title 1 - HTML body',
                'highlights': ['- highlight 1', '- highlight 2'],
                'body': 'Content 1: this is a <b>HTML string</b>',
                'footer': ['- footer 1a', '- <i>italic footer 1b</i>'],
                'show': True},
            {
                'title': 'Page Title 2 - No highlights',
                'body': 'Content 2: this is a <b>HTML string</b>',
                'footer': ['- Note: No highlights, so no lightbulb in the index page', '- <i>italic footer 2b</i>'],
                'show': True}
        ],
        'Subcat Y': [
            {
                'title': 'Page Title 3 - Plotly fig !',
                'highlights': ['- highlight 3', '- note: no footer on this page'],
                'body': simple_fig(),
                'show': True}
        ]
    },
    'Topcat B': {
        'Subcat Z': [
            {
                'title': 'Page Title 4 - Different topcat',
                'highlights': ['- highlight 5', '- highlight 6'],
                'body': 'Content 3',
                'footer': ['- footer 4a', '- footer 4b'],
                'show': True}
        ]
    }
}

# MAIN
ws.create(content=content
          , title_page=title_page
          , fname='webslides_hello_world.html'
          , open_in_browser=True
          , show_index_page=True
          , show_topcat=True
          , show_subcat=True
          , show_highlights_page=True
          , show_highlights_only=False)



import os
import sys
import codecs
import webbrowser

from webslides.modules.tohtml import *
from webslides.modules.generate import *
from webslides.modules.pagination import *
from webslides.modules.input_validations import *

def create(content=None
           , title_page=None
           , fname='output.html'
           , open_in_browser=True
           , show_index_page=False
           , show_highlights_page=False
           , show_topcat=True
           , show_subcat=True
           , show_highlights_only=False):
    """
    param pd (pagedata): list of lists with html strings or plotly fig objects ie.
        [[titlepage],[hl_page_title,hl_page_content],
        [index_page_title,index_page_content],
        [content_page1_title,content_page1_comments,content_page1_fig],
        [content_page2_title,content_page2_comments,content_page2_fig]]
    param fname: str filename for output file. If not provided defaults to 'output.html')
    param write: bool, False will only output to screen, True will write file AND output to screen
    return: str html code
    """

    # INPUT VALIDATIONS
    if title_page:
        title_error = validate_title_page(title_page)
        if title_error:
            print(title_error)
            return

    if content:
        content_error = validate_content(content)
        if content_error:
            print(content_error)
            return
    else:
        print('content variable is mandatory')

    # create dataframe from pagedata list and enrich with pagination data
    df = pagination_data(content=content, show_highlights_only=show_highlights_only)

    #################
    ## CREATE HTML ##
    #################

    # page shadow css options: https://getcssscan.com/css-box-shadow-examples
    html = """
    <html>
    <head><meta charset="utf-8" />
    <script src="https://cdn.plot.ly/plotly-latest.min.js"></script>
    <style>
    .page {padding:50px; margin:100px; box-shadow: rgba(0, 0, 0, 0.35) 0px 5px 15px;}
    body {font-family: "Arial";} a {text-decoration: none;}
    </style>
    </head>
    <body>
    <span></span>
    </body>
    </html>
    """

    # 1. title page
    if title_page:
        page = {'title': title_page.get('title', ''),
                'img_url': title_page.get('img_url', ''),
                'summary': title_page.get('summary', ''),
                'footer': title_page.get('footer', list())}

        # start new page
        html = html.replace('<span></span>', '<div class="page"><span></span></div>')

        # reuse contenpage_to_html function (normally used for contenslides)
        html = titlepage_to_html(html, page)

        # close page
        html = html.replace('<span></span></div>', '</div><span></span>')

    # 2. index page
    if show_index_page:
        # start new page
        html = html.replace('<span></span>', '<div class="page"><span></span></div>')

        indexpage_html = generate_index_page(df, show_topcat=show_topcat, show_subcat=show_subcat)
        html = html.replace('<span></span>', indexpage_html + '<span></span>')

        # close page
        html = html.replace('<span></span></div>', '</div><span></span>')

    # 3. highlights page
    if show_highlights_page:
        # start new page
        html = html.replace('<span></span>', '<div class="page"><span></span></div>')

        hlpage_html = generate_highlights_page(df, show_topcat=show_topcat, show_subcat=show_subcat)
        html = html.replace('<span></span>', hlpage_html + '<span></span>')

        # close page
        html = html.replace('<span></span></div>', '</div><span></span>')

    # 4. content pages
    for idx, page in df.iterrows():
        # start new page
        html = html.replace('<span></span>', '<div class="page"><span></span></div>')

        # insert content html
        html = content_to_html(html, page,
                               show_topcat=show_topcat,
                               show_subcat=show_subcat,
                               show_index_page=show_index_page,
                               show_highlights_page=show_highlights_page,
                               show_navi=True)

        # close page
        html = html.replace('<span></span></div>', '</div><span></span>')

    ###################
    ## HANDLE OUTPUT ##
    ###################

    # create output directory 'wsout' if not present
    current_working_dir = os.getcwd()
    fpath = os.path.join(current_working_dir, 'wsout')

    if not os.path.exists(fpath):
        os.makedirs(fpath)

    with codecs.open(f"{fpath}/{fname}", "w", encoding='utf-8') as f:
        f.write(html)
        print(f'output saved as wsout/{fname}')


    if open_in_browser:
        # open in browser to check result
        htmlfile = f"{fpath}/{fname}"
        webbrowser.open(htmlfile)
        print(f'opened in browser wsout/{fname}')

    return None




import pandas as pd


def pagination_data(content, show_highlights_only):
    """
    :param pagedata: list of dicts with config and contents per page
    :return: df with pagination data, like pagenumber, category, page order
    """

    data = list()
    for topcat, subcats in content.items():
        for subcat, items in subcats.items():
            for item in items:
                item_copy = item.copy()
                item_copy['topcat'] = topcat
                item_copy['subcat'] = subcat

                # Check for the presence of 'footer' and 'highlights' keys
                if 'footer' not in item_copy:
                    item_copy['footer'] = None
                if 'highlights' not in item_copy:
                    item_copy['highlights'] = None

                data.append(item_copy)

    df = pd.DataFrame(data)

    # only keep page to be shown
    # nb. if shown is not defined for any of the content pages all pages will be shown
    if 'show' in df.columns:
        df = df[df.show != False]

    df['pagekey'] = df.index
    df = df.reset_index(drop=True)

    if 'topcat' in df.columns:
        df['next_tcat'] = df.topcat.shift(1)
    if 'subcat' in df.columns:
        df['next_scat'] = df.subcat.shift(1)

    # determine page keys of previous/next highlight page
    if 'highlights' in df.columns:
        # extra column 'highlight' indicating if page contains any highlights
        df['highlight'] = df.apply(lambda x: True if x['highlights'] else False, axis=1)

    # pagenumber, +1 because index starts on 0
    df['pageno'] = df.index + 1

    if 'highlights' in df.columns:
        df['hl_page'] = df.highlight * df.pageno

        # # define previous highlight page (hl_prev)
        df['hl_max'] = df.hl_page.cummax()  # cum_max
        df['hl_prev'] = df.hl_max.shift(1)  # cum_max

        # # define next highlight page (hl_next)
        df['hl_page2'] = df.hl_page.replace(0, 99)
        df['hl_min'] = df.loc[::-1, 'hl_page2'].cummin()[::-1]
        df['hl_next'] = df.hl_min.shift(-1).astype('Int64')

        # fill invalid references to next and previous highlight
        # dit is geval voor eerste en laatste highlight
        df['hl_prev'] = df['hl_prev'].fillna(0)
        df['hl_next'] = df['hl_next'].fillna(0)

        # next en previous page key toevoegen aan next en previous page number
        pagekey = dict(zip(df.pageno, df.pagekey))
        pagekey[0] = ''
        pagekey[99] = ''
        df['hl_prev_key'] = df.apply(lambda x: pagekey[x['hl_prev']], axis=1)
        df['hl_next_key'] = df.apply(lambda x: pagekey[x['hl_next']], axis=1)

        # drop temp highlight columns
        df = df.drop(['hl_page', 'hl_max', 'hl_page2', 'hl_min', 'hl_next', 'hl_prev'], axis=1)

    # remove nans (occur if certain keys are missing in content pages)
    # replacing with '' allows to check if a value is present
    df = df.fillna('')

    return df



import datetime as dt
import plotly.offline as po


def titlepage_to_html(html, page):
    # check if image is provided, otherwise swap for Webslides logo HTML
    if ('img_url' in page) and len(page['img_url']) > 0:

        # image url was provided
        VAR_IMAGE_HTML = "<img src='" + page['img_url'] + "' style='width:200px;margin:10px;'>"

    else:

        # no image url provided
        VAR_IMAGE_HTML = ws_logo_html()

    # init
    titlepage = """<style>
    .container {position: relative; height: 600px;}
    .centered-element {text-align: center; margin: 0; width:80%; position: absolute; top: 50%; left: 50%; -ms-transform: translate(-50%, -50%); transform: translate(-50%, -50%);}
    </style>
    <div class='container'>
        <div class='centered-element'>
            VAR_IMAGE_HTML
            <h1>VAR_TITLE</h1><br>
            <p style='line-height: 1.5;'>
            VAR_SUMMARY</p>
        </div>
    </div>"""

    titlepage = titlepage.replace('VAR_IMAGE_HTML', VAR_IMAGE_HTML)
    titlepage = titlepage.replace('VAR_TITLE', page['title'])

    # generate summary html
    summary_html = """
        <table style="width: 100%; text-align: left; font-size: 1em; border-collapse: collapse; border: 0;">
            <colgroup>
                <col style="width: 20%;">
                <col style="width: 80%;">
            </colgroup>
            <tbody>"""

    for k, v in page['summary'].items():
        summary_html += f"""
                <tr style="border: 0;">
                    <td style="border: 0; padding: 10px; text-decoration: underline; vertical-align: top; text-align: right;">{k}</td>
                    <td style="border: 0; padding: 10px;">{v}</td>
                </tr>
                """
    summary_html += """
                <!-- more rows here -->
            </tbody>
        </table>"""

    titlepage = titlepage.replace('VAR_SUMMARY', summary_html)

    html = html.replace('<span></span>', titlepage + '<span></span>')

    # footer
    if 'footer' in page:
        footer = page['footer']
        footer.append(f"{dt.datetime.now().strftime('%e %b. %Y')}")
        footer_html = footer_to_html(page['footer'])
        html = html.replace('<span></span>', footer_html + '<span></span>')

    return html


def content_to_html(html, page, show_topcat, show_subcat, show_index_page=False, show_highlights_page=False,
                    show_navi=False):
    # A. page navigation
    if show_navi:  # dont show for title page
        pagenavi = pagenavi_to_html(pageno=page['pageno'],
                                    pagekey=page['pagekey'],
                                    show_index_page=show_index_page,
                                    show_highlights_page=show_highlights_page,
                                    prev_highlight_key=page.get('hl_prev_key', None),
                                    next_highlight_key=page.get('hl_next_key', None))
        html = html.replace('<span></span>', pagenavi + '<span></span>')

    # B. page title
    if 'title' in page and page['title'] != '':
        title = title_to_html(page['topcat'], page['subcat'], page['title'], show_topcat, show_subcat)
        html = html.replace('<span></span>', title + '<span></span>')

    # C. highlights
    if 'highlights' in page and page['highlights'] != '':
        highlights_html = highlights_to_html(page['highlights'])
        html = html.replace('<span></span>', highlights_html + '<span></span>')

    # D. body
    if 'body' in page:
        body_html = body_to_html(page['body'])
        html = html.replace('<span></span>', body_html + '<span></span>')

    # E. footer
    if 'footer' in page and page['footer'] != '':
        footer_html = footer_to_html(page['footer'])
        html = html.replace('<span></span>', footer_html + '<span></span>')

    return html


def pagenavi_to_html(pageno, show_index_page, show_highlights_page, pagekey, prev_highlight_key, next_highlight_key):
    """
    :param pageno: int pagenumber
    :param name: str name of the html page (id used to navigate to this page, ie. from index page)
    :param show_index_page: bool if False, references to index page are omitted
    :param show_highlights_page: bool if False, references to highlights page are omitted
    :param prev_highlight_key: str html id of previous highlight page
    :param next_highlight_key: str html id of next highlight page
    :return: string html code to place on top of each page, used for
        - navigation to next/previous highlight page
        - navigation to index page
        - navigation to highlights page
        - display pagenumber
        - page id tag used to navigate to that page
    """

    link_prev = '' if prev_highlight_key == '' else f'<a title="Previous highlight" href="#{prev_highlight_key}">&#9664;</a>'
    link_next = '' if next_highlight_key == '' else f'<a title="Next highlight" href="#{next_highlight_key}">&#9658;</a>'

    pagenavi_index_page = f'<a id="{pagekey}" title="Table of contents" href="#id_contents">&#128196;</a> ' * show_index_page
    pagenavi_highlights_page = f'{link_prev} <a title="Highlights summary" href="#highlights">&#128161;</a> {link_next} ' * show_highlights_page

    pagenavi_html = f'<div class="footer" style="width: 100%; margin: 0 auto; text-align:right;">{pagenavi_highlights_page}{pagenavi_index_page}p{pageno}</div>'

    return pagenavi_html


def title_to_html(topcat='', subcat='', title='', show_topcat=True, show_subcat=True):
    """
    :param title: str title of html page
    :return: string html formatted title
    """
    topcat = f"<span style='color: white; background-color: #008AC9; padding:5px;'>{topcat.upper()}</span> " if len(
        topcat) > 0 else ''
    subcat = f"<span style='font-weight: normal;'>{subcat}</span>: " if len(subcat) > 0 else ''
    return f"<h3 style='line-height: 2;'>{show_topcat * topcat}{show_subcat * subcat}{title}</h3>"


def highlights_to_html(highlights):
    """
    :param comments: list with text to show in header
    :return: string html formatted header
    """
    html = '<div style="background-color: #EBEBEB; line-height: 1.6; padding:10px;">'
    for o in highlights: html += f"{o}<br>"
    html += "</div>"
    return html


def body_to_html(body):
    # insert html string
    if isinstance(body, str):
        # add some padding to the body content
        return f"<div style='padding:3%'>{body}</div>"

    # if not string, must be plotly fig object
    else:
        return f"<div style='padding:3%'>{po.plot(body, include_plotlyjs=False, output_type='div')}</div>"


def footer_to_html(footer):
    """
    :param footer: list with text to show in footer
    :return: string html formatted footer
    """
    html = '<div style="background-color: #FFFFFF; line-height: 1.6; padding:10px;"><HR style="width:20%; margin-left:0;">'
    for o in footer: html += f"{o}<br>"
    html += "</div>"
    return html


def ws_logo_html():
    return """
        <style>
        .badge {
            width: 210px;
            height: 130px;
            margin: 42px auto;
            background: linear-gradient(to bottom right, white, #006fff);
            border: 3px solid #8ca5ff;
            border-radius: 10px;
            box-shadow: 2px 2px 5px rgb(0 0 0 / 30%);
            display: flex;
            justify-content: center;
            align-items: center;
            font-weight: bold;
            font-size: 60;
            color: #ededed;
        }
        </style>
        <div class="badge">
            <span>WS</span>
        </div>
    """



def generate_index_page(df, show_topcat, show_subcat):
    """
    :param df: dataframe with page meta data
    :return: list with index page content (list of html strings)
    """

    # init
    index_page_title = f"<h3 id='id_contents'>Contents</h3>"
    index_page_content = '<table style="line-height: 1.6;"><tr><td style="padding:20px; vertical-align: top;">'

    # calc for splitting index page contents into columns
    helft = max(15, int(df.shape[0] / 2))  # aantal items per html table kolom, maar tenminste 15
    derde = max(15, int(df.shape[0] / 2.7))  # aantal items per html table kolom, maar tenminste 15

    # import category descriptions (to show as tooltip infobutton)
    # cats = import_categories()

    # info button html
    ib_html = '<span title="VAR_CATEGORY" style="background: #4FB4E3; border-radius: 50%; height: 1.1em; width: 1.1em; text-align: center; color: #FFF; line-height: 1.1em; display:inline-block; font-weight: bold;">?</span>'

    # category
    for i, r in df.iterrows():

        # invoegen top category label
        if 'topcat' in df.columns and show_topcat:
            if r['topcat'] != r['next_tcat']:
                title = r['topcat']  # hier evt. nog een cat description toevoegen
                index_page_content += f"{'<br>' if i > 0 else ''}<span title='{title}' style='color: white; background-color: #008AC9; padding:5px;'>{r['topcat'].upper()}</span><br><br>"

        # ivoegen sub category label
        if 'subcat' in df.columns and show_subcat:
            if r['subcat'] != r['next_scat']:
                cat_description = ''  # cats.get(r['category'], None)  # check if category description is available
                info_button_category = ib_html.replace('VAR_CATEGORY', cat_description) if cat_description else ''
                index_page_content += f"<b>{r['subcat']}</b>&nbsp;&nbsp;{info_button_category}<br>"

        if (i + 1) % derde == 0:  # nieuwe table column
            index_page_content += f'</td><td style="padding:20px; vertical-align: top;">'

        if 'highlight' in df.columns:
            bulb = df[df.index == i].highlight.values[0]  # bullet = &#8226;
        else:
            bulb = False
        index_page_content += f"&nbsp;&nbsp;&nbsp;&nbsp;{i + 1}.&nbsp;<a href='#{r['pagekey']}'>{df[df.index == i].title.values[0]}</a> {'&#128161;' if bulb else ''}<br>"

    index_page_content += f"</td></tr></table>"

    return index_page_title + index_page_content


def generate_highlights_page(df, show_topcat, show_subcat):
    """
    :param df: dataframe with page meta data
    :return: list with highlights page content (list of html strings)
    """

    # init
    hl_page_title = f"<h3 id='highlights'>Highlights summary &#128161;</h3><span  style='line-height: 1.6;'>"
    hl_page_content = ''

    # add new line for each content page
    for i, r in df.iterrows():

        # invoegen top category label
        if 'topcat' in df.columns and show_topcat:
            if r['topcat'] != r['next_tcat']:
                hl_page_content += f"<br><span title='{r['topcat'].upper()}' style='color: white; background-color: #008AC9; padding:5px;'>{r['topcat'].upper()}</span><br><br>"

        # invoegen sub category label
        if 'subcat' in df.columns and show_subcat:
            if r['subcat'] != r['next_scat']:
                hl_page_content += f"<b>{r['subcat']}</b><br>"

        # show comments for highlighted titles
        if r['highlights']:
            for comment in r['highlights']:
                hl_page_content += f"&nbsp;&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a href='#{r['pagekey']}'>{r['title']}</a>: {comment}<br>"

    hl_page_content += '</span>'

    return hl_page_title + hl_page_content



import os
import numpy as np
import pandas as pd
import plotly.express as px

from webslides.main import create
from .other import hello_world_html, code_to_html


def demo_plotly_fig():
    # Load the iris dataset provided by the library
    df = px.data.iris()
    fig = px.scatter(df, x="sepal_width", y="sepal_length", color="species",
                     size='petal_length', hover_data=['petal_width'],
                     title="Iris Dataset: Sepal Dimensions and Petal Characteristics by Species")
    fig.update_layout(height=700)
    return fig


def demo_folium_html():
    fname = 'folium_demo.html'
    current_dir = os.path.dirname(os.path.abspath(__file__))
    fpath = os.path.join(current_dir, fname)
    with open(fpath, 'r') as f:
        demo_html = f.read()
    return demo_html


def demo_df_html():
    fname = 'df_demo.html'
    current_dir = os.path.dirname(os.path.abspath(__file__))
    fpath = os.path.join(current_dir, fname)
    with open(fpath, 'r') as f:
        df_html = f.read()
    return df_html


def demo():
    """Creates documentation explaining why and how to use this package"""

    title_page = {
        'img_url': '',
        'title': 'Webslides Documentation',
        'summary': {
            'About': 'Webslides is a Python package that creates HTML presentations <b>like this one</b>.<br>It enables easy sharing of Python-generated content such as charts, other visuals and data',
            'Contents': 'This presentation demonstrates features of the HTML format and should help you get started',
            'PyPi': '<a href="https://pypi.org/project/webslides/">https://pypi.org/project/webslides/</a>'}
    }

    content = {
        'WHY': {'Advantages over Powerpoint':
                    [{'title': 'Dynamic pagelength',
                      'highlights': ['- Pages can be as long (or short) as needed, like this one!'],
                      'body': 'Nothing much on this page really, that\'s why its so short 😉',
                      'footer': ['- and adding a footer can be usefull too..',
                                 '- ps. Have you noted the page navigation on the top right of the page?',
                                 '- &#x1F4A1;: this marks pages with a highlight. If a page has no highlights (remarks in grey area) it is not shown in the highlight summary page nor marked in the table of contents']},
                     {'title': 'Interactive content',
                      'highlights': [
                          '- Native Plotly graph ojects can be inserted as content, and they have many interactive features',
                          '- Try zooming in the by dragging a window in the plot. Or toggle visibility of a data series via the legend keys'],
                      'body': demo_plotly_fig(),
                      'footer': ['- Link to Plotly gallery <a href="https://plotly.com/python/">here</a>']},
                     {'title': 'Any HTML content will render',
                      'highlights': ['- For example this map'],
                      'body': demo_folium_html(),
                      'footer': [
                          '- this map was created with Folium (<a href="https://python-visualization.github.io/folium/">docs</a>)']},
                     {'title': 'Styled Pandas Dataframes',
                      'highlights': ['- Wow, notice that max value marked red',
                                     '- Not to mention the lowest value, marked green'],
                      'body': demo_df_html(),
                      'footer': [
                          '- styling of a pandas dataframe can be done by creating a styler object and convert that to html',
                          '- ie. styled_df = df.style.background_gradient(cmap=\'Blues\') and html = styled_df.to_html()']},
                     {'title': 'Super speed updates!',
                      'highlights': [
                          '- A key advantage of this presentation form may be that updates only require the <i>press of a button</i>'],
                      'body': 'Very often data, analysis conlusions and other content is revised in getting to a final version of a presentation. Since all is in python script, updates of the presentation only requires the press of the ▶ Run button.',
                      'footer': ['- and pagination will automatically adjust if new content is inserted']
                      }
                     ]
                },
        'HOW': {'Getting Started': [{
            'title': 'Install and Import',
            'highlights': ['- Nothing new here, good old pip install..'],
            'body': code_to_html(
                '#install package<br>pip install webslides<br><br>#import package<br>import webslides as ws')},
            {'title': 'Run Demo',
             'highlights': ['- ws.demo() will generate the html you are reading now'],
             'body': code_to_html(
                 '#import<br>import webslides as ws<br><br>#create and open this demo.html in browser<br>ws.demo()'),
             'footer': [
                 '- html output file, in this case \'demo.html\', is always saved to a directory \'wsout\'']},
            {'title': 'Hello World',
             'highlights': ['- basic example, demonstrating all available parameters'],
             'body': hello_world_html(),
             'footer': [
                 '- HTML allows to add javascript too, like the copy button. That\'s actually only one line of code']}
        ]
        }}

    create(content=content
           , title_page=title_page
           , fname='demo.html'
           , show_topcat=True
           , show_subcat=True
           , open_in_browser=True
           , show_index_page=True
           , show_highlights_page=True
           , show_highlights_only=False)

    return None

# demo()



from setuptools import setup, find_packages

setup(
    name="webslides",
    version="0.6.0",
    author="Derk-Jan Woltjer",
    author_email="derkjan.woltjer@gmail.com",
    description="Creates powerpoint like slides in HTML format. Including title page, index page and page navigation.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    licence="MIT License",
    packages=find_packages(),
    include_package_data=True,
    install_requires=["pandas", "plotly"]
)



import plotly.graph_objects as go


def validate_title_page(title_page):
    if not isinstance(title_page, dict):
        return "The title page must be a dictionary"
    elif 'title' not in title_page or not isinstance(title_page['title'], str):
        return "The title page must contain a 'title' key with a string value"
    elif 'img_url' in title_page and not isinstance(title_page['img_url'], str):
        return "The image url, if provided, must be string value"
    elif 'summary' in title_page and not isinstance(title_page['summary'], dict):
        return "The summary, if provided, must be dictionary"
    elif 'footer' in title_page and not isinstance(title_page['footer'], list):
        for footer_item in title_page['footer']:
            if not isinstance(footer_item, str):
                return "Each element of the 'footer' list, if present, must be a string"
    else:
        return None


def validate_content(content):
    if not isinstance(content, dict):
        return "Error: Input is not a dictionary."

    if len(content) < 1:
        return "Error: Dictionary must have at least 1 key-value pair."

    for topcat, subcats in content.items():
        if not isinstance(subcats, dict):
            return f"Error: '{topcat}' value is not a dictionary. Content: {str(subcats)[:13]}...{str(subcats)[-14:]}"

        if len(subcats) < 1:
            return f"Error: '{topcat}' subdictionary must have at least 1 key-value pair."

        for subcat, items in subcats.items():
            if not isinstance(items, list):
                return f"Error: '{subcat}' value is not a list. Content: {str(items)[:13]}...{str(items)[-14:]}"

            if len(items) < 1:
                return f"Error: '{subcat}' list must have at least 1 element."

            for item in items:
                if not isinstance(item, dict):
                    return f"Error: List element is not a dictionary. Content: {str(item)[:13]}...{str(item)[-14:]}"

                for key, value in item.items():
                    if key == 'title' and not isinstance(value, str):
                        return f"Error: 'title' value is not a string. Content: {str(value)[:13]}...{str(value)[-14:]}"
                    elif key in ['highlights', 'footer'] and not (
                            isinstance(value, list) and all(isinstance(v, str) for v in value)):
                        return f"Error: '{key}' value is not a list of strings. Content: {str(value)[:13]}...{str(value)[-14:]}"
                    elif key == 'body' and not (isinstance(value, str) or isinstance(value, go.Figure)):
                        return f"Error: 'body' value is not a string or a Plotly Figure object. Content: {str(value)[:13]}...{str(value)[-14:]}"
    return None



import os
import re


def extract_version(file_name: str) -> str:
    version_pattern = r'version="(\d+\.\d+\.\d+)"'
    version = None

    with open(file_name, "r") as file:
        file_contents = file.read()
        match = re.search(version_pattern, file_contents)
        if match:
            version = match.group(1)

    return version


def hello_world_html():
    fname = 'hello_world.py'
    current_dir = os.path.dirname(os.path.abspath(__file__))
    fpath = os.path.join(current_dir)
    with open(f"{fpath}/{fname}", 'r') as f:
        code_lines = f.readlines()

    html_lines = [
        '<div style="font-size:1.1em;padding:15px;background-color:#fdffc7; display: inline-block;"><pre><code>']
    for line in code_lines:
        line = line.replace('import main as ws', 'import webslides as ws')
        html_line = line.rstrip().replace('<', '&lt;').replace('>', '&gt;')
        html_lines.append(html_line)
    html_lines.append('</code></pre></div>')

    # add copy button
    html_lines.append(
        """<br><br><button style="font-size: 1.3em; background-color: #00bb00;color: white;border-color: #c7c7c7 !important; border-radius: 6px;" onclick="navigator.clipboard.writeText(`""" + '\n'.join(
            html_lines[1:-1]) + """`);alert('Code copied to clipboard');"><b>COPY CODE</b></button>""")

    return '\n'.join(html_lines)


def code_to_html(code):
    # return f'<div style="font-family: Courier New; font-weight: normal; background-color:#fdffc7; display: inline-block; padding:15px;">{code}</div>'
    return f'<div style="background-color:#fdffc7; display: inline-block;font-size:1.1em;padding:15px;"><pre><code>{code}</code></pre></div>'
